#!/usr/bin/perl

use strict;
use coqm;
use Socket;

use Getopt::Std;

my %opts;

getopts( 'htvui:', \%opts );

my $arp_table;

if ( $opts{ 'h'} || ! $ARGV[1] )
{
	print "usage: $0 [-htvu] [-i ifindex] community_string router ...\n";
	print "uses snmp to retrieve arp table from router(s)\n";
	print "options:\n";
	print " -h = print this message\n";
	print " -v = verbose mode\n";
	print " -t = print unix epoch timestamp with arp entries\n";
	print " -u = use ucd/net-snmp snmpwalk command instead of perl-snmp module to walk tables\n";
	print " -i ifindex = retrieve only arp entries for given ifindex\n";
	
}

my $community_string = shift(@ARGV);

foreach my $router (@ARGV)
{
	my $rh_session = coqm::create_session( $router, $community_string );

	if ( $opts{ 'u' } )
	{
		$$rh_session{ 'use_ucd_snmpwalk' } = "YES";
	}
	my $arp_table = coqm::fetch_arp_table($opts{ 'i' });
	$arp_table = $$arp_table{ 'by_ip' };

	my $timestamp = time();
	foreach my $ip (sort (keys %$arp_table))
	{
		if( ! $opts{ 't' } )
		{
			printf "%16s  %16s\n", $ip, $$arp_table{ $ip };
		}
		else
		{
			printf "%16s %16s  %16s\n", $timestamp, $ip, $$arp_table{ $ip };
		}
	}
	
}
