#!/usr/bin/perl

use strict;
use coqm;
use Socket;

use Getopt::Std;

my %opts;

getopts( 'hvuta:d:', \%opts );

if ( $opts{ 'h' } ||  ! $ARGV[0] ) { usage(); }


my $community = shift(@ARGV);

my @devices = (@ARGV);
if ( $opts{ 'd' } )
{
	open(FILE, $opts{ 'd' } ) || die "failed to open file \"$opts{ 'd' }\" for reading\n";
	foreach my $line (<FILE>)
	{
		chomp($line);
		if ( $line =~ /^\s*$/ || $line =~ /^#/ ) { next; }
		push( @devices, $line );
	}
}

foreach my $device (@devices)
{

	my $rh_session = coqm::create_session( $device, $community );
	my $rh = coqm::fetch_ifName();

	# if we don't get ifName, use ifDescr
	if ( scalar( (keys %{ $rh } ) ) < 1 )
	{
		$rh = coqm::fetch_ifDescr();
	}

	if ( $opts{ 'u' } )
	{
		$$rh_session{ 'use_ucd_snmpwalk' } = "YES";
	}

	my $time = time();

	my $cam_table = coqm::fetch_cam_table();
	$cam_table = $$cam_table{ 'by_ifindex' };

	foreach my $ifindex (keys %$cam_table)
	{
		#print "$ifindex , $$rh{ $ifindex } ,  $$rh2{ $ifindex }\n";
		my $ifname = $$rh{ $ifindex };
		if ( ! $ifname && ! $opts{ 'v' } ) { next };
		my $ra_macs = $$cam_table{ $ifindex };
		foreach my $mac ( @$ra_macs )
		{
			$opts{ 't' } || printf ("%-16s %-16s %-16s\n", $device, $ifname,$mac );
			$opts{ 't' } && printf ("%-16s %-16s %-16s %-16s\n", $time, $device, $ifname,$mac );
		}
	}
}

sub usage
{
	print "usage: $0 [-hut] [-d device_file] community_string device ...\n";
	print "retrieves cam table for device(s) via snmp.\n";
	print "options:\n";
	print "-h = show this message\n";
	print "-t = print unix epoch time with cam entries\n";
	print "-u = use ucd/net-snmp snmpwalk program do the walk (used for devices that don't hand back cam/arp tables in numerical order)\n";
	print "-d device_file = loads list of devices to query from device_file, format is one device per line\n";
	exit 1;
}




sub load_links
{
	my ($dbfile) = (@_); 
	my %info;
	open( FILE, $opts{ 'l' } ) || print "Could not open \"$dbfile\" for reading\n";
	foreach my $line (<FILE>)
	{
		chomp($line);
		if ( $line =~ /^$/ || $line =~ /^#/ ) { next; }
		my ($left, $right ) = split( / \-\> /, $line, 2 );
		my ( $local_switch, $local_port ) = split( " ", $left, 2 );
		my ( $remote_switch, $remote_port ) = split( " ", $right, 2 );
		if ( $remote_port eq "mac addrs" ) { next; }
		$info{ 'by_switch' }{ $local_switch }{ $remote_switch } = $local_port;
		$info{ 'by_port' }{ $local_switch }{ $local_port } = $remote_switch;
	}
	close(FILE);
	return \%info;
}
