#! /usr/bin/perl
use strict;
use coqm;
use Getopt::Std;

my %opts;

getopts( 'hvfi:', \%opts );


my $interface = lc($opts{ 'i' });
my $check_stp = $opts{ 'f' };


my $community = shift(@ARGV);


if ( $opts{ 'h' } || $ARGV[0] eq "" ) { usage(); }

foreach my $device (@ARGV)
{

my $rh_session = coqm::create_session($device, $community );



my $cdpGlobalDeviceId = coqm::fetch_cdpGlobalDeviceId();
printf "%-28s %-30s\n", "IP: $ARGV[1]", "DeviceID: $cdpGlobalDeviceId";


# In order to get StpState information, we will need a bridge port to ifindex table
my $rh_bridgeport_to_ifindex;
my $rh_ifindex_to_bridgeport;
if ( $opts{ 'f' } )
{
	$rh_bridgeport_to_ifindex = coqm::fetch_bridgeport_to_ifindex();
	foreach my $bridgeport ( keys %$rh_bridgeport_to_ifindex )
	{
		my $ifindex = $$rh_bridgeport_to_ifindex{ $bridgeport };
		$$rh_ifindex_to_bridgeport{ $ifindex } = $bridgeport;
	}
}

my $rh_ifDescr = coqm::fetch_ifDescr();

foreach my $ifindex (sort { $a <=> $b } (keys %{$rh_ifDescr}) )
{
	my $ifDescr = $$rh_ifDescr{ $ifindex };
	my $ifName = coqm::fetch_ifName( $ifindex );

	if ( $interface && $interface ne lc($ifDescr) && $interface ne lc($ifName) ) { next; }

	my $ifOperStatus = coqm::fetch_ifOperStatus( $ifindex );
	my $ifAdminStatus = coqm::fetch_ifAdminStatus( $ifindex );
	my $ifSpeed = commify( coqm::fetch_ifSpeed( $ifindex ) );
	
	printf "%-28s %-30s\n", "ifDesc: $ifDescr", "ifName: $ifName";
	printf "\t%-20s %-20s %-7s%13s\n", "AdminStatus: $ifAdminStatus",  "OperStatus: $ifOperStatus", "Speed: ", $ifSpeed;

	
	my $ifAccessVlan = coqm::fetch_ifAccessVlan( $ifindex );

	my $vlanTrunkPortDynamicState = coqm::fetch_vlanTrunkPortDynamicState( $ifindex );
	my $vlanTrunkPortEncapsulationType = coqm::fetch_vlanTrunkPortEncapsulationType( $ifindex );
	my $vlanTrunkPortDynamicStatus = coqm::fetch_vlanTrunkPortDynamicStatus( $ifindex );
	my $vlanTrunkPortEncapsulationOperType = coqm::fetch_vlanTrunkPortEncapsulationOperType( $ifindex );
	my $vlanTrunkPortNativeVlan = coqm::fetch_vlanTrunkPortNativeVlan( $ifindex );

	printf "\t%-20s %-20s\n", "Access Vlan: $ifAccessVlan",  "Trunking Config: $vlanTrunkPortDynamicState, $vlanTrunkPortEncapsulationType";
	printf "\t%-20s %-20s\n", "Native Vlan: $vlanTrunkPortNativeVlan",  "Trunking Status: $vlanTrunkPortDynamicStatus, $vlanTrunkPortEncapsulationOperType";

	if ( $vlanTrunkPortDynamicState ne "access" && $vlanTrunkPortDynamicState ne ""  )
	{
		my $ra_vlans = coqm::fetch_vlanTrunkPortVlansEnabled( $ifindex );
		my $vlans = coqm::consolidate_list($ra_vlans);
		printf "\t%-s\n", "Vlans Enabled: $vlans";

		if ( $opts{ 'f' } )
		{
			# STP stuff
			my $bridgeport = $$rh_ifindex_to_bridgeport{ $ifindex };
			my $rh_dot1dStpPortState = coqm::fetch_dot1dStpPortState( $bridgeport, $ra_vlans);
			my %StpStates;
			foreach my $vlan ( keys %$rh_dot1dStpPortState )
			{
				my $StpState = $$rh_dot1dStpPortState{ $vlan };
				if ( $StpState eq "NULL" ) { next; }
				push( @{$StpStates{ $StpState }}, $vlan );
			}
			foreach my $StpState ( sort (keys %StpStates) )
			{
				my $vlans = coqm::consolidate_list( $StpStates{ $StpState } );
				print "\tSTP $StpState: $vlans\n";
			}
		}
	}

	# does the device support cdp?
	if ( $cdpGlobalDeviceId )
	{
		my $rh_nAddress = coqm::fetch_cdpCacheAddress( $ifindex );
		foreach my $pid (sort{ $a <=> $b } keys ( %$rh_nAddress ) )
		{
			my $nAddress = $$rh_nAddress{ $pid };
			my $nDeviceId = coqm::fetch_cdpCacheDeviceId( $ifindex, $pid );
			my $nDevicePort = coqm::fetch_cdpCacheDevicePort( $ifindex, $pid );
			my $nDevicePlatform = coqm::fetch_cdpCachePlatform( $ifindex, $pid );
			my $nNativeVLAN = coqm::fetch_cdpCacheNativeVLAN( $ifindex, $pid );
			my $nCacheDuplex = coqm::fetch_cdpCacheDuplex( $ifindex, $pid );
			printf "\tNeighbor: $nAddress $nDeviceId $nDevicePort $nDevicePlatform\n";
		}
	}
	print "\n";
}

}

sub commify {
    my $text = reverse $_[0];
    $text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
    return scalar reverse $text;
}


sub usage
{
	print "usage: $0 [-vf] [-i interface] community_string device ...\n";
	print "Uses SNMP to retrieve interesting information about port(s) on device(s)\n";
	print "options:\n";
	print "-v = verbose\n";
	print "-f = report on stp forwarding state\n";
	print "-i interface = only retrieve information for interface\n";

}
